@@ -15,7 +15,7 @@ from botocore.exceptions import ClientError, NoCredentialsError
 from rich import box
 from rich.console import Console
 from rich.panel import Panel
-from rich.progress import Progress, SpinnerColumn, TextColumn
+from rich.progress import Progress, SpinnerColumn, TextColumn, TimeElapsedColumn
 from rich.prompt import Confirm, IntPrompt, Prompt
 from rich.table import Table
 import re
@@ -1314,6 +1314,40 @@ def create_ami(
                 )
                 console.print(f"  Excluding volume at {device_name}")
 
+        # --- Check & detach attached studios --------------------------------------------------
+        # If any user studios are still attached we must detach them before the instance reboots
+        # for snapshot consistency; otherwise Studio-Manager metadata becomes stale.
+
+        attached_resp = make_api_request("GET", f"/engines/{engine['instance_id']}/studios")
+        attached_studios = attached_resp.json().get("studios", []) if attached_resp.status_code == 200 else []
+
+        if attached_studios:
+            console.print(f"[yellow]⚠  {len(attached_studios)} studio(s) attached to this engine.[/yellow]")
+            for s in attached_studios:
+                console.print(f"  • {s['user']} ({s['studio_id']})")
+
+            if not Confirm.ask("Detach all studios before AMI creation?"):
+                console.print("AMI creation cancelled.")
+                return
+
+            console.print("Detaching studios…")
+            for s in attached_studios:
+                resp = make_api_request("POST", f"/studios/{s['studio_id']}/detach")
+                if resp.status_code != 200:
+                    console.print(f"[red]❌ Failed to detach {s['studio_id']} – aborting.[/red]")
+                    return
+
+            # Wait briefly for volumes to become available (max 2 min)
+            # (time is already imported at module level)
+            ec2_wait = boto3.client("ec2", region_name="us-east-1")
+            vol_ids = [s['studio_id'] for s in attached_studios]
+            console.print("Waiting for volumes to detach…")
+            waiter = ec2_wait.get_waiter("volume_available")
+            try:
+                waiter.wait(VolumeIds=vol_ids, WaiterConfig={"Delay": 5, "MaxAttempts": 24})
+            except Exception:
+                console.print("[yellow]Proceeding even though some volumes may still be detaching.[/yellow]")
+
         # Create the AMI
         with Progress(
             SpinnerColumn(),
@@ -1594,9 +1628,28 @@ def attach_studio(
                 raise typer.Exit(1)
             console.print("[green]✓ Engine started[/green]")
             console.print("Waiting for engine to be ready...")
-            import time
+            # Wait until the engine is fully running & marked ready (max 5 min)
+            console.print("Waiting for engine to be ready (this can take a couple of minutes)…")
+            # Progress classes already imported at module level – do not re-import to avoid UnboundLocalError
 
-            time.sleep(10)
+            with Progress(SpinnerColumn(), TimeElapsedColumn(), TextColumn("[progress.description]{task.description}"), transient=True) as prog:
+                prog.add_task("Awaiting engine readiness…", total=None)
+                max_attempts = 30  # 30 × 10 s  ≈ 5 min
+                for _ in range(max_attempts):
+                    time.sleep(10)
+
+                    status_resp = make_api_request("GET", f"/engines/{engine['instance_id']}")
+                    if status_resp.status_code != 200:
+                        continue
+
+                    detailed = status_resp.json().get("engine", {})
+                    state_ok = detailed.get("state", "").lower() == "running"
+                    ready_ok = detailed.get("ready") is True
+
+                    if state_ok and ready_ok:
+                        break
+                else:
+                    console.print("[yellow]Engine is still starting up. If attachment fails, wait a little longer and retry.[/yellow]")
         else:
             raise typer.Exit(1)
 
